--------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------                                                                                 VARIABLES                                                                                                 ---------
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------

--- Load from cfg
function readData(f)
	local settings = util.KeyValuesToTable(f) 

	local intParams = {
		"plague_active",
		"plague_headcrabs",
		"plague_notify",
		"plague_particles",
		"plague_sent_activetoggle",
		"plague_sent_antivirus",
	}
	
	local stringParams = {
		"plague_msg_on",
		"plague_msg_off",
		"plague_msg_anti",
		"plague_type",
		"plague_access"
	}

	
	local mycvars = {
		"pvar_sent_health",
		"pvar_npcsleep",
		"pvar_pln",
		"pvar_fst",
		"pvar_pzn",
		"pvar_r",
		"pvar_g",
		"pvar_b"
	}
	
	for k,v in pairs(intParams) do
		if settings[v] then
			SetGlobalInt(v, tonumber(settings[v]))
		end
	end
	
	for k,v in pairs(stringParams) do
		if settings[v] then
			SetGlobalString(v, settings[v])
		end
	end
	
	for k,v in pairs(mycvars) do
		if settings[v] then
			CreateConVar(v, settings[v], FCVAR_UNREGISTERED)
		end
	end
end

-- save current settings
function saveSettings(ply, cmd, args)
	if !ply:IsAdmin() then 
		ply:PrintMessage(HUD_PRINTCONSOLE, "Error:  you do not have access to this command")
		ply:PrintMessage(HUD_PRINTCENTER, "INSUFFICIENT ACCESS")
		return 
	end 
	
	local path = args[1]
	
	if !path then
		path = "plague/userdefault.txt"
	
	if #args > 1 then
		ply:PrintMessage(HUD_PRINTCONSOLE, "Error:  too many args")
		return
	end
	
	elseif !string.find(path,".txt") then
		path = path .. ".txt"
	end
	
	if !string.find(path,"plague/") then
		path = "plague/" .. path
	end
	
	local data = { '"PlagueUser"', '{',
	    '\t"plague_active"\t"' .. GetGlobalInt("plague_active") .. '"',
		'\t"plague_headcrabs"\t"' .. GetGlobalInt("plague_headcrabs") .. '"',
		'\t"plague_notify"\t"' .. GetGlobalInt("plague_notify") .. '"',
		'\t"plague_particles"\t"' .. GetGlobalInt("plague_particles") .. '"',
		'\t"pvar_npcsleep"\t"' .. GetConVarString("pvar_npcsleep") .. '"',
		'\t"pvar_r"\t"' .. GetConVarString("pvar_r") .. '"',
		'\t"pvar_g"\t"' .. GetConVarString("pvar_g") .. '"',
		'\t"pvar_b"\t"' .. GetConVarString("pvar_b") .. '"',
		'\t"pvar_pln"\t"' .. GetConVarString("pvar_pln") .. '"',
		'\t"pvar_fst"\t"' .. GetConVarString("pvar_fst") .. '"',
		'\t"pvar_pzn"\t"' .. GetConVarString("pvar_pzn") .. '"',
		'\t"plague_sent_activetoggle"\t"' .. GetGlobalInt("plague_sent_activetoggle") .. '"',
		'\t"plague_sent_antivirus"\t"' .. GetGlobalInt("plague_sent_antivirus") .. '"',
		'\t"pvar_sent_health"\t"' .. GetConVarString("pvar_sent_health") .. '"',
		'\t"plague_msg_on"\t"' .. GetGlobalString("plague_msg_on") .. '"',
		'\t"plague_msg_off"\t"' .. GetGlobalString("plague_msg_off") .. '"',
		'\t"plague_msg_anti"\t"' .. GetGlobalString("plague_msg_anti") .. '"',
		'\t"plague_type"\t"' .. GetGlobalString("plague_type") .. '"',
		'\t"plague_access"\t"' .. GetGlobalString("plague_access") .. '"',
		'}'
	}
	
	local f = table.concat(data, '\n')
	file.Write(path, f)
	
	ply:PrintMessage(HUD_PRINTCONSOLE, "Saved to '" .. path .."'\n")
	ply:PrintMessage(HUD_PRINTCENTER, "Saved!")
end

concommand.Add("plague_save",		saveSettings)

if SERVER then 
	-- load user pref first
	local f = file.Read("plague/userdefault.txt")
	-- otherwise default
	if !f then  f = file.Read("../cfg/plague_defaults.txt") end
	
	readData(f)
	local txt = {}
	txt[1] = "ON"
	txt[0] = "OFF"
	
	Msg("\n*****************\n*  Plague loaded and is " .. txt[GetGlobalInt("plague_active")] .. "\n*****************\n\n")

end

if SERVER then
	CreateConVar("pvar_savefile", "", FCVAR_UNREGISTERED)
	CreateConVar("pvar_loadfile", "", FCVAR_UNREGISTERED)
	
	function pvarSave(ply, cmd, args)
		if !ply:IsAdmin() then 
			ply:PrintMessage(HUD_PRINTCENTER, "INSUFFICIENT ACCESS")
			return 
		end 
		
		saveSettings(ply, cmd, { GetConVarString("pvar_savefile") })
		ply:PrintMessage(HUD_PRINTCENTER, "SAVED!")
	end
	
	function pvarLoad(ply, cmd, args)
		if !ply:IsAdmin() then 
			ply:PrintMessage(HUD_PRINTCENTER, "INSUFFICIENT ACCESS")
			return 
		end 
		
		loadSettings(ply, cmd, { GetConVarString("pvar_loadfile") })
		ply:PrintMessage(HUD_PRINTCENTER, "SAVED!")
	end
	
	concommand.Add("pvar_save", pvarSave)
	concommand.Add("pvar_load", pvarSave)
end

