
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------                                                                          LOAD SOUNDS                                                                                                  ---------
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------

util.PrecacheSound("npc/zombie/zombie_alert1.wav")
util.PrecacheSound("npc/fast_zombie/fz_alert_close1.wav")
util.PrecacheSound("npc/zombie_poison/pz_alert1.wav")

SetGlobalInt("corpse_color_r", 0)
SetGlobalInt("corpse_color_g", 0)
SetGlobalInt("corpse_color_b", 0)

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------                                                                          MAIN FUNCTIONS                                                                                             ---------
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------

if SERVER then
	deadNPC = {}
	corpses = {}
end

// make a player a zombie
function playerZombify(victim, Inflictor, killer )
	-- quit if you are a client or there is no plague
	if (!SERVER || GetGlobalInt("plague_active") == 0) then return end

	-- is the killer an NPC
	if !killer:IsNPC() then return end
	
	-- create the entity
	local zombie = makeZombie(killer) 
	
	-- if there is a new zombie
	if zombie then
		-- place npc
		zombie:SetPos(victim:GetPos())
		zombie:SetAngles(Angle(0,victim:GetAngles().y,0))
	
		-- prepare for player spawn
		victim.zombie = zombie
	end
end

//   Create zombie on player respawn
function spawnPlyZombie (ply)
	if (!SERVER) then return end
	
	-- if player was zombified
	if ply.zombie then
		ply.zombie:Spawn()
		ply.zombie:Activate()
		SetGlobalInt("corpse_color_r",GetConVarNumber("pvar_r"))
		SetGlobalInt("corpse_color_g",GetConVarNumber("pvar_g"))
		SetGlobalInt("corpse_color_b",GetConVarNumber("pvar_b"))
		
		-- create particles
		if GetGlobalInt("plague_particles") == 1 then
			local ed = EffectData()
			ed:SetOrigin( ply.zombie:GetPos() )
			ed:SetEntity( ply.zombie )
			util.Effect( "zombie_spawn", ed )
		end
		
		-- play sound
		local snd = getSound(ply.zombie:GetClass())
		if snd then
			ply.zombie:EmitSound(snd, 100, 100)
		end
	end

	-- destroy reference
	ply.zombie = nil
end


// Create zombie from NPC
function npcZombify(victim, killer, ent )
	-- quit if you are a client or there is no plague
	if (!SERVER || GetGlobalInt("plague_active") == 0) then return end

	-- NPC's that can become zombies
	local zombNPC = {
		"npc_mossman",
		"npc_monk",
		"npc_metropolice",
		"npc_kleiner",
		"npc_gman",
		"npc_eli",
		"npc_combine_s",
		"npc_citizen",
		"npc_breen",
		"npc_barney",
		"npc_alyx",
		"npc_stalker",
		"npc_vortigaunt"
	}
	-- is victim legitimate
	if !table.HasValue(zombNPC, victim:GetClass()) then return end
	
	local zombie = makeZombie(killer) -- create the entity
	
	-- if there is a new zombie
	if zombie then
		-- remove old npc
		local Entry = {
				zombie = zombie,
				pos = victim:GetPos() }
		
		table.insert(deadNPC, Entry)
	end
end

-- handle getting new corpses and processing old corpses
function npcCorpses()
	// get new corpses
	if deadNPC && #deadNPC > 0 then
		for k,npc in pairs(deadNPC) do
			if npc then
				// find ragdoll
				local ents = ents.FindInSphere(npc.pos, .0001)
				for a, ent in pairs(ents) do
					if ent && ent:IsValid() && ent:GetClass() == "prop_ragdoll" && 
					   !table.HasValue(corpses,tostring(ent)) then
						ent.zombie = npc.zombie
						ent.diedAt = CurTime()
						ent:SetColor(GetConVarNumber("pvar_r"), GetConVarNumber("pvar_g"), GetConVarNumber("pvar_b"), 255)
						table.insert(corpses, ent)
					end
				end
			end
		
			deadNPC[k] = nil
		end
	end
	
	// process existing corpses
	if corpses && #corpses > 0 then
		for k,body in pairs(corpses) do
			if body then
				// is it ripe?
				if CurTime() >= body.diedAt+GetConVarNumber("pvar_npcsleep") then
					local zombie = body.zombie
					color = {body:GetColor()}
					SetGlobalInt("corpse_color_r",color[1])
					SetGlobalInt("corpse_color_g",color[2])
					SetGlobalInt("corpse_color_b",color[3])
					
					-- get corpse angle and pos
					zombie:SetPos(body:GetPos())
					zombie:SetAngles(Angle(0,body:GetAngles().y,0))
					
					--particles
					if GetGlobalInt("plague_particles") == 1 then						
						local ed = EffectData()
						ed:SetOrigin(zombie:GetPos())
						ed:SetEntity(zombie)
						util.Effect( "zombie_spawn", ed )
					end
				
					--play sound
					local snd = getSound(zombie:GetClass())
					if snd then
						zombie:EmitSound(snd, 100, 100)
					end
					
					-- remove corpse
					body:Remove()
					
					-- spawn
					zombie:Spawn()
					zombie:Activate()
					
					-- cleanup
					corpses[k] = nil
				end
			end
		end
	end
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------                                                                     HELPER FUNCTIONS                                                                                             ---------
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------

// create zombie npc
function makeZombie(killer) 
	-- get class that killed PC/NPC
	local class = killer:GetClass()
	
	-- check if a valid zombie
	if (class=="npc_fastzombie" || class=="npc_zombie" || class=="npc_poisonzombie" || class=="npc_zombie_torso") || 
	  (string.find(class,"npc_headcrab") == 1 && GetGlobalInt("plague_headcrabs") == 1) then  -- do we count headcrabs?
			
		-- get type to spawn
		zombie = getType()
		
		-- if nil then create the same kind that killed the PC/NPC
		if !zombie then
			if string.find(class, "fast") then
				zombie.type = "npc_fastzombie"
			elseif string.find(class, "poison") then
				zombie = "npc_poisonzombie"
			else
				zombie = "npc_zombie"
			end
		end
		
		-- return new zombie
		return ents.Create(zombie)
	end
end


// a method to find out what type of zombie should be spawned
function getType()
	-- init variables
	local zombietype = "npc_zombie"
	local plaguetype = GetGlobalString("plague_type")
	
	-- randomly set it by ch variables
	if (plaguetype == "rand") then
		local ch_zomb = GetConVarNumber("pvar_pln")
		local ch_fast = GetConVarNumber("pvar_fst")
		local ch_pozn = GetConVarNumber("pvar_pzn")

		local ch_total = ch_zomb + ch_fast + ch_pozn
		
		local choice = math.Rand(0,ch_total)
		if choice < ch_zomb then
			zombietype = "npc_zombie"
		elseif choice < ch_zomb + ch_fast then
			zombietype = "npc_fastzombie"
		else
			zombietype = "npc_poisonzombie"
		end
		
	-- nil prompts function to copy previous type
	elseif plaguetype == "same" then
		zombietype = nil
		
	-- basic zombie types
	elseif plaguetype == "plain" then
		zombietype = "npc_zombie"
	elseif plaguetype == "fast" then
		zombietype = "npc_fastzombie"
	elseif plaguetype == "poison" then
		zombietype = "npc_poisonzombie" 
	end
	
	-- return
	return zombietype
end


// get spawn sound
function getSound(zombietype)
	if zombietype == "npc_zombie" then
		return "npc/zombie/zombie_alert1.wav"
	elseif zombietype == "fast" then
		return "npc/fast_zombie/fz_alert_close1.wav"
	elseif zombietype == "poison" then
		return "npc/zombie_poison/pz_alert1.wav"
	end
end

-- print the state of the plague
function plagueUpdate()
	-- pick message
	local msg = ""
	local snd = ""
	if GetGlobalInt("plague_active") == 1 then
		msg = GetGlobalString("plague_msg_on")
	else
		msg = GetGlobalString("plague_msg_off")
	end
	
	notify(msg, snd)
end


--------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------                                                                            PLAYER NOTIFY                                                                                               ---------
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------

-- print the state of the plague
function notify(msg, snd)
	-- send all players the message
	for k, ply in pairs(player.GetAll()) do		
		if GetGlobalInt("plague_notify") == 1 then 
			Msg(msg .. "\n")
			ply:PrintMessage(HUD_PRINTCENTER, msg)
		end
	end
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------                                                                                        HOOKS                                                                                                  ---------
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------

hook.Add("PlayerDeath", "player_zombify", playerZombify)
hook.Add("OnNPCKilled", "npc_zombify", npcZombify)
hook.Add("PlayerSpawn", "player_spawn_zombie", spawnPlyZombie)
hook.Add("Think", "npc_corpses", npcCorpses)

--------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------                                                                                        HOOKS                                                                                                  ---------
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function getNPC()
	for k,ragdoll in pairs(ents.FindByClass("npc_*")) do
		if ragdoll and ragdoll:IsValid() then
			//this is a hacky method to check if a player made the ragdoll or the engine did
			//for some reason the engine makes ragdolls with /'s in the modelpath
			//and gmod spawns then with \'s in the path

			Msg(tostring(ragdoll) .. ", " .. tostring(ragdoll:GetPos()) .. "\n")
		end
	end
end
